﻿using System;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.RegularExpressions;

namespace PACNET
{
    /// <summary>Error Handling APII</summary>
    public class ErrHandling
    {
        //11.1. PAC_API void pac_GetErrorMessage(DWORD dwMessageID, LPTSTR lpBuffer)
        /// <summary>
        /// This function retrieves a message string.
        /// </summary>
        /// <param name="dwMessageID">[in] Specifies the 32-bit message identifier for the requested message.</param>
        /// <param name="lpBuffer">[out] Pointer to a buffer for the error message.</param>
        [DllImport("PACSDK.dll", EntryPoint = "pac_GetErrorMessage")]
        public extern static void GetErrorMessage(uint dwMessageID, string lpBuffer);

        /// <summary>
        /// Get a message string  
        /// </summary>
        /// <param name="dwMessageID">Specifies the 32-bit message identifier for the requested message  </param>
        /// <returns>Error message string  </returns>
        public static string GetErrorMessage(uint dwMessageID)
        {
            string errMessage = new string(' ', 512);

            PACNET.ErrHandling.GetErrorMessage(dwMessageID, errMessage);

            return errMessage.Split('\0')[0];
        }

        //11.2. PAC_API DWORD pac_GetLastError()
        /// <summary>
        /// This function returns the last-error code value.
        /// </summary>
        /// <returns>The Return Value section of each reference page notes the conditions under which the
        /// function sets the last-error code.</returns>      
        [DllImport("PACSDK.dll", EntryPoint = "pac_GetLastError")]
        public extern static uint GetLastError();

        //11.3. PAC_API void pac_SetLastError(DWORD errno)
        /// <summary>
        /// This function sets the last-error code.
        /// </summary>
        /// <param name="errno">[in] Specifies the last-error code.</param>
        [DllImport("PACSDK.dll", EntryPoint = "pac_SetLastError")]
        public extern static void SetLastError(uint errno);

        ///////////////////////////////////////////////////////////////////////
        //
        // from Xpac
        //
        ///////////////////////////////////////////////////////////////////////

        //8.4. PAC_API void pac_ClearLastError(DWORD errorno)
        /// <summary>
        /// This function clears the last-error code.
        /// </summary>
        [DllImport("PACSDK.dll", EntryPoint = "pac_ClearLastError")]
        public extern static void ClearLastError();
    }

    /// <summary>Error Codes</summary>
    public enum ErrCode
    {
        /// <summary>No Error</summary>
        PAC_ERR_SUCCESS = 0x00000,

        /// <summary>Unknow Error</summary>
        PAC_ERR_UNKNOWN = 0x00001,


        ///////////////////////////////////////////////////////////////////////
        // Basic Error Codes
        ///////////////////////////////////////////////////////////////////////

        /// <summary>Slot registered error</summary>
        PAC_ERR_SLOT_REGISTERED = 0x10001,

        /// <summary>Slot not registered error</summary>
        PAC_ERR_SLOT_NOT_REGISTERED = 0x10002,

        /// <summary>Unknown Module</summary>
        PAC_ERR_UNKNOWN_MODULE = 0x10003,

        /// <summary>Module doesn't exist</summary>
        PAC_ERR_INVALID_MAC = 0x10004,

        /// <summary>Invalid COM port number</summary>
        PAC_ERR_INVALID_COMPORT_NUMBER = 0x10005,

        /// <summary>Function not supported</summary>
        PAC_ERR_FUNCTION_NOT_SUPPORT = 0x10006,

        /// <summary>Module doesn't exist</summary>
        PAC_ERR_MODULE_UNEXISTS = 0x10007,

        /// <summary>Slot not registered error</summary>
        PAC_ERR_INVALID_SLOT_NUMBER = 0x10008,


        ///////////////////////////////////////////////////////////////////////
        // Memory Error Codes
        ///////////////////////////////////////////////////////////////////////

        /// <summary>EEPROM accesses invalid address</summary>
        PAC_ERR_EEP_INVALID_ADDRESS = 0x11001,

        /// <summary>SRAM accesses invalid address</summary>
        PAC_ERR_SRAM_INVALID_ADDRESS = 0x11002,

        /// <summary>SRAM accesses invalid type</summary>
        PAC_ERR_MEMORY_INVALID_TYPE = 0x11003,

        /// <summary>NVRAM accesses invalid address</summary>
        PAC_ERR_NVRAM_INVALID_ADDRESS = 0x11004,

        /// <summary>EEPROM write protection</summary>
        PAC_ERR_EEP_WRITE_PROTECT = 0x11005,

        /// <summary>EEPROM write fail</summary>
        PAC_ERR_EEP_WRITE_FAIL = 0x11006,

        /// <summary>EEPROM read fail</summary>
        PAC_ERR_EEP_READ_FAIL = 0x11007,


        ///////////////////////////////////////////////////////////////////////
        // WatchDog Error Codes
        ///////////////////////////////////////////////////////////////////////

        /// <summary>The input value is invalid</summary>
        PAC_ERR_WDT_INVALID_VALUE = 0x12001,

        /// <summary>The wdt doesn't exist</summary>
        PAC_ERR_WDT_UNEXIST = 0x12002,

        /// <summary>The wdt init error</summary>
        PAC_ERR_WDT_BASE_NOT_SET = 0x12003,


        ///////////////////////////////////////////////////////////////////////
        // Interrupt Error Codes
        ///////////////////////////////////////////////////////////////////////

        /// <summary>Create interrupt's event failure</summary>
        PAC_ERR_INTR_CREATE_EVENT_FAILURE = 0x13001,


        ///////////////////////////////////////////////////////////////////////
        // UART Error Codes
        ///////////////////////////////////////////////////////////////////////

        /// <summary>Uart check sum error</summary>
        PAC_ERR_UART_CHECKSUM = 0x14001,

        /// <summary>Uart read timeout</summary>
        PAC_ERR_UART_READ_TIMEOUT = 0x14002,

        /// <summary>Uart response error</summary>
        PAC_ERR_UART_RESPONSE = 0x14003,

        /// <summary>Uart under input range</summary>
        PAC_ERR_UART_UNDER_INPUT_RANGE = 0x14004,

        /// <summary>Uart exceed input range</summary>
        PAC_ERR_UART_EXCEED_INPUT_RANGE = 0x14005,

        /// <summary>Uart open filed</summary>
        PAC_ERR_UART_OPEN_FAILED = 0x14006,

        /// <summary>Uart get Comm Modem status error</summary>
        PAC_ERR_UART_GET_COMM_STATUS_ERROR = 0x14007,

        /// <summary>Uart get wrong line status</summary>
        PAC_ERR_UART_INVALID_VALUE = 0x14008,

        /// <summary>Uart internal buffer overflow</summary>
        PAC_ERR_UART_INTERNAL_BUFFER_OVERFLOW = 0x14009,


        ///////////////////////////////////////////////////////////////////////
        // IO Error Codes
        ///////////////////////////////////////////////////////////////////////

        /// <summary>IO card does not support this API function</summary>
        PAC_ERR_IO_NOT_SUPPORT = 0x15001,

        /// <summary>API unsupport this IO card</summary>
        PAC_ERR_IO_ID = 0x15002,

        /// <summary>Slot's value exceeds its range</summary>
        PAC_ERR_IO_SLOT = 0x15003,

        /// <summary>Channel's value exceeds its range</summary>
        PAC_ERR_IO_CHANNEL = 0x15004,

        /// <summary>Gain's value exceeds its range</summary>
        PAC_ERR_IO_GAIN = 0x15005,

        /// <summary>Unsupported interrupt mode</summary>
        PAC_ERR_IO_INT_MODE = 0x15006,

        /// <summary>I/O value is out of the range</summary>
        PAC_ERR_IO_VALUE_OUT_OF_RANGE = 0x15007,

        /// <summary>I/O channel is out of the range</summary>
        PAC_ERR_IO_CHANNEL_OUT_OF_RANGE = 0x15008,

        /// <summary>DI/O channel can't overwrite</summary>
        PAC_ERR_IO_DO_CANNOT_OVERWRITE = 0x1500A,

        /// <summary>AI/O channel can't overwrite</summary>
        PAC_ERR_IO_AO_CANNOT_OVERWRITE = 0x1500B,


        ///////////////////////////////////////////////////////////////////////
        // Backplane Timer Error Codes
        ///////////////////////////////////////////////////////////////////////

        /// <summary>Backplane Timer registed</summary>
        PAC_ERR_BPTIMER_REGISTERED = 0x16001,

        /// <summary>Backplane Timer not registed</summary>
        PAC_ERR_BPTIMER_NOT_REGISTERED = 0x16002,
    }
}